/*
 * $Id: workbenchprefs.c,v 1.1.1.1 2005/03/15 16:01:01 laire Exp $
 */
#define AROS_ALMOST_COMPATIBLE
#include "global.h"
#include <workbench/icon.h>
#include <graphics/view.h>
#include <proto/iffparse.h>
#include <proto/icon.h>
#include <aros/asmcall.h>

#define DEBUG 0
#include <aros/debug.h>

extern struct Library		*WorkbenchBase;
extern struct Library		*IconBase;

static BOOL FirstCall = TRUE;


#define WB35PSIZE  offsetof(struct WorkbenchPrefs, wbp_ImageMemType)
#define WB39PSIZE  sizeof(struct WorkbenchPrefs)

AROS_UFH1(LONG, WorkbenchPrefs_Handler,
		  AROS_UFHA(struct IFFHandle *, iff, A2))
{
	struct WorkbenchPrefs wbprefs =
	{
		8192,			/* wbp_DefaultStackSize */
		3,				/* wbp_TypeRestartTime */
		PRECISION_ICON,	/* wbp_IconPrecision */
		{-1, -1, 1, 1},	/* wbp_EmbossRect */
		FALSE,			/* wbp_Borderless */
		107-6,			/* wbp_MaxNameLength */
		TRUE,			/* wbp_NewIconsSupport */
		TRUE,			/* wbp_ColorIconSupport */
		/* WB 3.9+ */
		MEMF_PUBLIC,	/* wbp_ImageMemType */
		FALSE,			/* wbp_LockPens */
		FALSE,			/* wbp_NoTitleBar */
		FALSE			/* wbp_NoGauge */
	};

	if (ReadChunkBytes(iff, &wbprefs, sizeof(wbprefs)) < 0)
	{
		return 1;
	}

	/* WB 3.5 stuff */
	if (IconBase->lib_Version >= 44)
	{
		IconControl(NULL,
		            ICONCTRLA_SetGlobalPrecision,        wbprefs.wbp_IconPrecision,
		            ICONCTRLA_SetGlobalEmbossRect,       (ULONG) &wbprefs.wbp_EmbossRect, 
		            ICONCTRLA_SetGlobalFrameless,        wbprefs.wbp_Borderless,
		            ICONCTRLA_SetGlobalMaxNameLength,    wbprefs.wbp_MaxNameLength,
		            ICONCTRLA_SetGlobalNewIconsSupport,  wbprefs.wbp_NewIconsSupport,
		            ICONCTRLA_SetGlobalColorIconSupport, wbprefs.wbp_ColorIconSupport,
		            TAG_DONE);
	}

	/* WB 3.9 stuff */
	if (IconBase->lib_Version >= 45)
	{
		IconControl(NULL,
		            ICONA_Reserved1, wbprefs.wbp_ImageMemType,
		            TAG_DONE);
	}

	/* WB 3.5 stuff */
	if (WorkbenchBase->lib_Version >= 44)
	{
		WorkbenchControl(NULL,
		                 WBCTRLA_SetDefaultStackSize, wbprefs.wbp_DefaultStackSize,
		                 WBCTRLA_SetTypeRestartTime,  wbprefs.wbp_TypeRestartTime,
		                 WBA_Reserved11,              TRUE,
		                 TAG_DONE);
	}

	/* WB 3.9 stuff */
	if (WorkbenchBase->lib_Version >= 45)
	{
		BOOL ChangePens, SomeFlag = FALSE;
		static const struct
		{
			UWORD pen;
			ULONG r, g, b;
		} PenTable[5] = 
		{
			{4, 0x79797979, 0x79797979, 0x79797979},
			{5, 0xaeaeaeae, 0xaeaeaeae, 0xaeaeaeae},
			{6, 0xaaaaaaaa, 0x92929292, 0x7d7d7d7d},
			{7, 0xffffffff, 0xaaaaaaaa, 0x96969696},
			{256, 0, 0, 0}
		};
		LONG PrevGauge;

		ChangePens = PrevLockPens != wbprefs.wbp_LockPens;
		PrevLockPens = wbprefs.wbp_LockPens;

		WorkbenchControl(NULL,
		                 WBA_Reserved15, (ULONG) &PrevGauge,
		                 TAG_DONE);

		WorkbenchControl(NULL,
		                 WBA_Reserved5,  wbprefs.wbp_ImageMemType,
		                 WBA_Reserved21, wbprefs.wbp_NoTitleBar ? TRUE : FALSE,
		                 WBA_Reserved14, wbprefs.wbp_NoGauge ? TRUE : FALSE,
		                 ChangePens ? WBA_Reserved18 : TAG_IGNORE, wbprefs.wbp_LockPens ? (ULONG) PenTable : 0,
		                 TAG_DONE);

		if (ChangePens && PrevLockPens && !FirstCall)
		{
			SomeFlag = TRUE;
		}

		if (PrevGauge == wbprefs.wbp_NoGauge && !SomeFlag)
		{
			struct List *opendrawerlist = NULL;

			if (WorkbenchControl(NULL,
			                     WBCTRLA_GetOpenDrawerList, (ULONG) &opendrawerlist,
			                     TAG_DONE) &&
			    opendrawerlist)
			{
				if (!IsListEmpty(opendrawerlist))
				{
					struct Node *Node;

					ForeachNode(opendrawerlist, Node)
					{
						if (PrevGauge || Node->ln_Name[strlen(Node->ln_Name) - 1] == ':')
						{
							CloseWorkbenchObjectA(Node->ln_Name, NULL);
						}
					}

					ForeachNode(opendrawerlist, Node)
					{
						if (PrevGauge || Node->ln_Name[strlen(Node->ln_Name) - 1] == ':')
						{
							OpenWorkbenchObjectA(Node->ln_Name, NULL);
						}
					}
				}

				WorkbenchControl(NULL,
				                 WBCTRLA_FreeOpenDrawerList, (ULONG) opendrawerlist,
				                 TAG_DONE);
			}
		}

		FirstCall = FALSE;
	}

	return 0;

	AROS_LIBFUNC_EXIT
}

AROS_UFH1(LONG, WorkbenchHiddenDevicePrefs_Handler,
		  AROS_UFHA(struct IFFHandle *, iff, A2))
{
	struct ContextNode *chunk;
	struct Node *newnode;

	chunk = CurrentChunk(iff);

	newnode = AllocVec(sizeof(struct Node) + chunk->cn_Size, MEMF_ANY);
	if (newnode)
	{
		UBYTE *buf = ((UBYTE *) newnode) + sizeof(struct Node);

		newnode->ln_Type = 0;
		newnode->ln_Pri  = 0;
		newnode->ln_Name = buf;
		if (ReadChunkBytes(iff, buf, chunk->cn_Size) == chunk->cn_Size)
		{
			ADDTAIL(&HiddenDeviceList, newnode);
		}
		else
		{
			FreeVec(newnode);
			return IoErr();
		}
	}

	return 0;

	AROS_LIBFUNC_EXIT
}
